#include "Globals.h"
#include "DeskbarView.h"
#include <String.h>

#include <Entry.h>
#include <Bitmap.h>
#include <Roster.h>
#include <NodeInfo.h>
#include <PopUpMenu.h>
#include <MenuItem.h>
#include <Deskbar.h>
#include <iostream.h>
#include <NetEndpoint.h>
#include <Messenger.h>
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
int32 RetrieveFromBeBits(void *msngr)
{
  BString reply;
  BString Software;
  BMessage *msg = NULL;
  BNetEndpoint *endpoint = new BNetEndpoint();
  if( B_OK != endpoint->Connect("www.bebits.com",80) )
    return B_ERROR;
  endpoint->Send(request, 51 );
  reply.UnlockBuffer( endpoint->Receive( reply.LockBuffer(BUFFER_SIZE), BUFFER_SIZE ) );
  int32 offset1 = reply.FindLast("%%"); reply.RemoveLast("%%");
  int32 offset2 = reply.FindLast("%%");
  while( offset1 > 0 && offset2 > 0 )
  {
    reply.MoveInto(Software, offset2 + 2, offset1 - offset2 - 2  );
    msg = new BMessage(BEBITS_UPDATE);
    BString name,url;
    Software.RemoveFirst("\n");
    Software.ReplaceFirst("\n"," - ");
    Software.MoveInto(name, 0, Software.FindFirst("\n") ); 
    Software.RemoveFirst("\n");
    Software.MoveInto(url    , 0, Software.FindFirst("\n") ); 
    msg->AddString("name"   ,name   );
    msg->AddString("be:url" ,url    );
    (*(BMessenger*)msngr).SendMessage(msg,msg);
    delete msg;
    offset1 = reply.FindLast("%%"); reply.RemoveLast("%%");
    offset2 = reply.FindLast("%%");
  }
  return B_OK;
}
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
BBitmap *get_resource_bitmap()
{
  entry_ref	ref;
  BBitmap *bmp = new	BBitmap(BRect(0,0, B_MINI_ICON-1, B_MINI_ICON-1), B_CMAP8);	
  if (be_roster->FindApp(GLOBAL_APP_SIG, &ref)!=B_OK) return NULL;
  BNode      file( &ref );
  BNodeInfo  *info = new BNodeInfo(&file);
  info->GetIcon(bmp, B_MINI_ICON);
  delete info;
  return bmp;
}
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
BView *instantiate_deskbar_item()
{ return new DeskbarView(); }
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
DeskbarView::DeskbarView()
    : BView( BRect(0, 0, 15, 15), VIEW_NAME, B_FOLLOW_LEFT_RIGHT | B_FOLLOW_TOP_BOTTOM, B_WILL_DRAW | B_PULSE_NEEDED)
{}
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
DeskbarView::DeskbarView(BMessage *archive)
	: BView(archive),
	  count( 0 ),
	  new_item (false)
{}
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void DeskbarView::AttachedToWindow(void) 
{
	SetViewColor(Parent()->ViewColor());
	SetDrawingMode( B_OP_ALPHA );
	Bitmap = get_resource_bitmap();
	BMessage *menu_msg2 = new BMessage(GOTO_URL);
	BMessage *menu_msg3 = new BMessage(CHECK_NOW);
	menu_msg2->AddString("be:url","http://www.bebits.com");
	BMenuItem *item1 = new BMenuItem( "Quit"                 , new BMessage(B_QUIT_REQUESTED) );
	BMenuItem *item2 = new BMenuItem( "GoTo BeBits.com"      , menu_msg2 );
	BMenuItem *item3 = new BMenuItem( "Check Now For Updates", menu_msg3 );
	item1->SetTarget(this);
    item2->SetTarget(this);
    item3->SetTarget(this);
    menu = new BPopUpMenu("Menu");
    menu->SetRadioMode(false);
    menu->AddItem( item1 );
    menu->AddItem( item2 );
    menu->AddItem( item3 );
    menu->AddSeparatorItem();
}
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void DeskbarView::DetachedFromWindow(void) 
{
  delete Bitmap;
  delete menu;
}
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void DeskbarView::Pulse()
{
  if( new_item ) { SetDrawingMode(B_OP_INVERT); DrawBitmap( Bitmap ); }
  if( count++ % MOD_VALUE == 0  )
  { CheckForUpdates(); }
}
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void DeskbarView::Draw(BRect rect)
{
  SetDrawingMode( B_OP_ALPHA );
  DrawBitmap( Bitmap );
}
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void DeskbarView::AddToDeskbar()
{
	BDeskbar *deskbar = new BDeskbar();
	entry_ref ref;
	be_roster->FindApp(GLOBAL_APP_SIG, &ref);
	deskbar->AddItem(&ref);
    delete deskbar;
}
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void DeskbarView::RemoveFromDeskbar()
{
	BDeskbar deskbar;
	deskbar.RemoveItem(VIEW_NAME);
}
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void DeskbarView::MouseDown(BPoint p)
{
  BPoint cursor; uint32 buttons;
  GetMouse(&cursor, &buttons);
  if(B_SECONDARY_MOUSE_BUTTON == buttons)
  { ConvertToScreen(&p); menu->Go(p,true,false,true); }
  if( new_item ) { new_item = false; Invalidate(); }  
}
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
BArchivable *DeskbarView::Instantiate(BMessage *data)
{ return new DeskbarView(data); }
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
status_t DeskbarView::Archive(BMessage *data, bool deep) const
{
	BView::Archive(data, deep);
	data->AddString("add_on", GLOBAL_APP_SIG);
	data->AddString("class", VIEW_NAME);
	return B_OK;
}
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void DeskbarView::MessageReceived(BMessage *msg)
{
  switch(msg->what)
  {
    case BEBITS_UPDATE:
    {
      BString name, url;
      msg->FindString("name"   , &name   );
      msg->FindString("be:url" , &url    );
      if( NULL == menu->FindItem(name.String() ) )
      {
        new_item = true;
        BMessage *menu_msg = new BMessage();
        *menu_msg = *msg; menu_msg->what = GOTO_URL;
        BMenuItem *item = new BMenuItem( name.String(), menu_msg );
        item->SetTarget( this );    
        menu->AddItem( item, 4 );
        while( menu->CountItems() > LIST_ITEM_COUNT + 4 )
        { delete menu->RemoveItem(LIST_ITEM_COUNT + 4 - 1); }
      }
    }
    break;
    case GOTO_URL:
    {
      BMessage *menu_msg = new BMessage();
      *menu_msg = *msg; menu_msg->what = BROWSER_OPEN_URL;
      be_roster->Launch(BROWSER_APP_SIGNATURE, menu_msg);
    }
    break;
    case CHECK_NOW:
      CheckForUpdates();
    break;
    case B_QUIT_REQUESTED:
      RemoveFromDeskbar();
  } 
}
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void DeskbarView::CheckForUpdates()
{ resume_thread( spawn_thread(RetrieveFromBeBits, "Checking BeBits for updates..", B_NORMAL_PRIORITY, new BMessenger(this) ) ); }
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
